import pytest

from .utils import run_make


@pytest.mark.parametrize(
    "wrapper",
    [{"E3_MODULE_SRC_PATH": "test-loc"}],
    indirect=True,
)
def test_loc(wrapper):
    rc, _, errs = run_make(wrapper, "build", E3_MODULE_SRC_PATH="test-loc")
    assert rc == 2
    assert 'Local source mode "-loc" has been deprecated' in errs


def test_sitelibs(wrapper):
    # Overwrite the default makefile
    with open(wrapper / "Makefile", "w") as f:
        f.write(
            """
TOP:=$(CURDIR)
include $(REQUIRE_CONFIG)/RULES_E3
include $(REQUIRE_CONFIG)/RULES_CELL
include $(REQUIRE_CONFIG)/DEFINES_FT
include $(REQUIRE_CONFIG)/RULES_PATCH
include $(REQUIRE_CONFIG)/RULES_TEST

include $(REQUIRE_CONFIG)/RULES_DKMS
include $(REQUIRE_CONFIG)/RULES_VARS

include $(REQUIRE_CONFIG)/RULES_DEV
"""
        )
    rc, _, errs = run_make(wrapper, "build")
    assert rc == 2
    assert "RULES_E3 should only be loaded from RULES_SITEMODS" in errs


def test_incorrect_module_name(wrapper):
    module_name = "ADCore"
    rc, _, errs = run_make(wrapper, "build", E3_MODULE_NAME=module_name)
    assert rc == 2
    assert f"E3_MODULE_NAME '{module_name}' is not valid" in errs
