import re

import pytest

from .utils import TEST_MODULE_NAME, run_ioc_get_output, run_make

RE_MODULE_LOADED = f"Loaded {TEST_MODULE_NAME} version {{version}}"
RE_MODULE_NOT_LOADED = f"Module {TEST_MODULE_NAME} (not available|version {{required}} not available|version {{required}} not available \\(but other versions are available\\))"


@pytest.mark.parametrize(
    "wrapper",
    [{"templates": "", "sources": "", "dbds": "test.dbd", "headers": ""}],
    indirect=True,
)
@pytest.mark.parametrize(
    "requested, expected, installed",
    [
        # If nothing is installed, nothing should be loaded
        ("", "", []),
        # Test versions can be loaded
        ("test", "test", ["test", "0.0.1"]),
        # Numeric versions should be prioritized over test versions
        ("", "0.0.1+0", ["test", "0.0.1"]),
        ("", "0.0.1+0", ["0.0.1", "test"]),
        # Highest build number should be loaded if version is unspecified
        ("", "0.0.1+7", ["0.0.1", "0.0.1+7", "0.0.1+3"]),
        # Only load the given build number if it is specified
        ("0.0.1+0", "", ["0.0.1+1"]),
        # If no build number is specified, load the highest build number
        ("0.0.1", "0.0.1+4", ["0.1.0", "0.0.1+4", "0.0.1"]),
        # Build number 0 means load that version exactly
        ("0.0.1+0", "0.0.1+0", ["0.0.1+0"]),
        ("0.0.1+0", "0.0.1+0", ["0.0.1", "0.0.1+1", "1.0.0"]),
        # 1-test counts as a test version, as does 1.0
        ("", "0.0.1+0", ["0.0.1", "1-test"]),
        ("", "0.0.1+0", ["0.0.1", "1.0"]),
        # Numeric version should be prioritised over "higher" test version
        ("", "0.1.0+0", ["0.1.0", "1.0.0-rc1"]),
    ],
)
def test_version(wrapper, requested, expected, installed):
    for version in installed:
        returncode, _, _ = run_make(
            wrapper,
            "clean",
            "cellinstall",
            E3_MODULE_VERSION=version,
        )
        assert returncode == 0

    rc, o, e = run_ioc_get_output(requested, wrapper / "cellMods")

    if expected:
        match = re.search(RE_MODULE_LOADED.format(version=re.escape(expected)), o)
        assert rc == 0
        assert match
    else:
        match = re.search(RE_MODULE_NOT_LOADED.format(required=re.escape(requested)), o)
        assert match
        assert rc != 0
